@@ -399,6 +399,7 @@ class URDF:
         load_collision_meshes: bool = False,
         filename_handler=None,
         mesh_dir: str = "",
+        force_mesh: bool = False,
     ):
         """A URDF model.
 
@@ -410,6 +411,7 @@ class URDF:
             load_collision_meshes (bool, optional): Whether to load the collision meshes referenced in the <mesh> elements. Defaults to False.
             filename_handler ([type], optional): Any function f(in: str) -> str, that maps filenames in the URDF to actual resources. Can be used to customize treatment of `package://` directives or relative/absolute filenames. Defaults to None.
             mesh_dir (str, optional): A root directory used for loading meshes. Defaults to "".
+            force_mesh (bool, optional): Each loaded geometry will be concatenated into a single one (instead of being turned into a graph; in case the underlying file contains multiple geometries). This might loose texture information but the resulting scene graph will be smaller. Defaults to False.
         """
         if filename_handler is None:
             self._filename_handler = partial(filename_handler_magic, dir=mesh_dir)
@@ -431,14 +433,18 @@ class URDF:
 
         if build_scene_graph:
             self._scene = self._create_scene(
-                use_collision_geometry=False, load_geometry=load_meshes
+                use_collision_geometry=False,
+                load_geometry=load_meshes,
+                force_mesh=force_mesh,
             )
         else:
             self._scene = None
 
         if build_collision_scene_graph:
             self._scene_collision = self._create_scene(
-                use_collision_geometry=True, load_geometry=load_collision_meshes
+                use_collision_geometry=True,
+                load_geometry=load_collision_meshes,
+                force_mesh=force_mesh,
             )
         else:
             self._scene_collision = None
@@ -497,6 +503,15 @@ class URDF:
         """
         return self._actuated_joints
 
+    @property
+    def actuated_dof_indices(self):
+        """List of DOF indices per actuated joint. Can be used to reference configuration.
+
+        Returns:
+            list[list[int]]: List of DOF indices per actuated joint.
+        """
+        return self._actuated_dof_indices
+
     @property
     def actuated_joint_indices(self):
         """List of indices of all joints that are actuated, i.e., not of type mimic or fixed.
@@ -526,13 +541,13 @@ class URDF:
 
     @property
     def num_dofs(self):
-        """Number of degrees of freedom. Depending on the type of the joint, the number of DOFs might vary.
+        """Number of degrees of freedom of actuated joints. Depending on the type of the joint, the number of DOFs might vary.
 
         Returns:
             int: Degrees of freedom.
         """
         total_num_dofs = 0
-        for j in self.robot.joints:
+        for j in self._actuated_joints:
             if j.type in ["revolute", "prismatic", "continuous"]:
                 total_num_dofs += 1
             elif j.type == "floating":
@@ -559,13 +574,13 @@ class URDF:
         """
         config = []
         config_names = []
-        for j in self.robot.joints:
+        for j in self._actuated_joints:
             if j.type == "revolute" or j.type == "prismatic":
                 if j.limit is not None:
                     cfg = [j.limit.lower + 0.5 * (j.limit.upper - j.limit.lower)]
                 else:
                     cfg = [0.0]
-            elif j.type == "continuous" or j.type == "fixed":
+            elif j.type == "continuous":
                 cfg = [0.0]
             elif j.type == "floating":
                 cfg = [0.0] * 6
@@ -665,12 +680,28 @@ class URDF:
     def _update_actuated_joints(self):
         self._actuated_joints = []
         self._actuated_joint_indices = []
+        self._actuated_dof_indices = []
 
+        dof_indices_cnt = 0
         for i, j in enumerate(self.robot.joints):
             if j.mimic is None and j.type != "fixed":
                 self._actuated_joints.append(j)
                 self._actuated_joint_indices.append(i)
 
+                if j.type in ["prismatic", "revolute", "continuous"]:
+                    self._actuated_dof_indices.append([dof_indices_cnt])
+                    dof_indices_cnt += 1
+                elif j.type == "floating":
+                    self._actuated_dof_indices.append(
+                        [dof_indices_cnt, dof_indices_cnt + 1, dof_indices_cnt + 2]
+                    )
+                    dof_indices_cnt += 3
+                elif j.type == "planar":
+                    self._actuated_dof_indices.append(
+                        [dof_indices_cnt, dof_indices_cnt + 1]
+                    )
+                    dof_indices_cnt += 2
+
     def _validate_required_attribute(self, attribute, error_msg, allowed_values=None):
         if attribute is None:
             self._errors.append(URDFIncompleteError(error_msg))
@@ -693,6 +724,7 @@ class URDF:
             **load_collision_meshes (bool, optional): Whether to load the collision meshes referenced in the <mesh> elements. Defaults to False.
             **filename_handler ([type], optional): Any function f(in: str) -> str, that maps filenames in the URDF to actual resources. Can be used to customize treatment of `package://` directives or relative/absolute filenames. Defaults to None.
             **mesh_dir (str, optional): A root directory used for loading meshes. Defaults to "".
+            **force_mesh (bool, optional): Each loaded geometry will be concatenated into a single one (instead of being turned into a graph; in case the underlying file contains multiple geometries). This might loose texture information but the resulting scene graph will be smaller. Defaults to False.
 
         Raises:
             ValueError: If filename does not exist.
@@ -792,7 +824,7 @@ class URDF:
 
         return link_names[0]
 
-    def _forward_kinematics_joint(self, joint, q=0.0):
+    def _forward_kinematics_joint(self, joint, q=None):
         origin = np.eye(4) if joint.origin is None else joint.origin
 
         if joint.mimic is not None:
@@ -802,10 +834,19 @@ class URDF:
                 + joint.mimic.offset
             )
 
-        if joint.type == "revolute":
-            matrix = origin @ tra.rotation_matrix(q, joint.axis)
-        elif joint.type == "prismatic" or joint.type == "continuous":
-            matrix = origin @ tra.translation_matrix(q * joint.axis)
+        if joint.type in ["revolute", "prismatic", "continuous"]:
+            if q is None:
+                # Use internal cfg vector for forward kinematics
+                q = self.cfg[
+                    self.actuated_dof_indices[
+                        self.actuated_joint_names.index(joint.name)
+                    ]
+                ]
+
+            if joint.type == "prismatic":
+                matrix = origin @ tra.translation_matrix(q * joint.axis)
+            else:
+                matrix = origin @ tra.rotation_matrix(q, joint.axis)
         else:
             # this includes: floating, planar, fixed
             matrix = origin
@@ -852,7 +893,9 @@ class URDF:
             matrix, joint_q = self._forward_kinematics_joint(j, q=q)
 
             # update internal configuration vector - to enable mimic joint mapping
-            self._cfg[self.joint_names.index(j.name)] = joint_q
+            self._cfg[
+                self.actuated_dof_indices[self.actuated_joint_names.index(j.name)]
+            ] = joint_q
 
             if self._scene is not None:
                 self._scene.graph.update(
@@ -896,7 +939,9 @@ class URDF:
                     0
                 ]
 
-    def _add_visual_to_scene(self, s, v, link_name, load_geometry=True):
+    def _add_visual_to_scene(
+        self, s, v, link_name, load_geometry=True, force_mesh=False
+    ):
         origin = v.origin if v.origin is not None else np.eye(4)
 
         if v.geometry is not None:
@@ -926,11 +971,23 @@ class URDF:
                     _logger.debug(
                         f"Loading {v.geometry.mesh.filename} as {new_filename}"
                     )
-                    # try:
-                    # os.path.join(self.mesh_dir, v.geometry.mesh.filename)
-                    new_s = trimesh.load(
-                        new_filename, ignore_broken=True, force="scene"
-                    )
+
+                    if force_mesh:
+                        new_g = trimesh.load(
+                            new_filename, ignore_broken=True, force="mesh"
+                        )
+
+                        # add original filename
+                        if "file_path" not in new_g.metadata:
+                            new_g.metadata["file_path"] = os.path.abspath(new_filename)
+                            new_g.metadata["file_name"] = os.path.basename(new_filename)
+
+                        new_s = trimesh.Scene()
+                        new_s.add_geometry(new_g)
+                    else:
+                        new_s = trimesh.load(
+                            new_filename, ignore_broken=True, force="scene"
+                        )
 
                     # scale mesh appropriately
                     if v.geometry.mesh.scale is not None:
@@ -962,12 +1019,13 @@ class URDF:
                         transform=origin @ new_s.graph.get(name)[0],
                     )
 
-    def _create_scene(self, use_collision_geometry=False, load_geometry=True):
+    def _create_scene(
+        self, use_collision_geometry=False, load_geometry=True, force_mesh=False
+    ):
         s = trimesh.scene.Scene(base_frame=self._base_link)
 
-        configuration = self._cfg
-        for j, q in zip(self.robot.joints, configuration):
-            matrix, _ = self._forward_kinematics_joint(j, q=q)
+        for j in self.robot.joints:
+            matrix, _ = self._forward_kinematics_joint(j)
 
             s.graph.update(frame_from=j.parent, frame_to=j.child, matrix=matrix)
 
@@ -977,7 +1035,11 @@ class URDF:
             meshes = l.collisions if use_collision_geometry else l.visuals
             for m in meshes:
                 self._add_visual_to_scene(
-                    s, m, link_name=l.name, load_geometry=load_geometry
+                    s,
+                    m,
+                    link_name=l.name,
+                    load_geometry=load_geometry,
+                    force_mesh=force_mesh,
                 )
 
         return s
